
/*
	HE_Dos.h

	This is C code
	everywhere a:
	HEDskVolInfo& heVol= *pVolInfo;
	found is deleted in the port

	This stuff is colored with HEDsk_,
	defines with HE, no namespace in C

*/

//Offsets MBR
#define MBR_LBA0		0x01c6

//Offsets vol info
#define HEFAT_RESERVED		14 //0x0e WORD
#define HEFAT_SECPERFAT		22 //0x16 WORD
#define HEFAT_DIRENTRIES		26 //0x1a WORD //?Number of heads?
#define HEFAT_SECPERCLST		13 //0x0d BYTE

//Sizes
#define HEFAT_SZF_FNAME		11
#define HEFAT_SZF_DIRENT		32
#define HEFAT_ENTRIESPERSEC	16

//internal specific
#define HEDS_SECTORSIZE			512
#define HEDS_HALFSECMASK		0xff //255
#define HEDS_SECTOR_SHIFT		9
#define HEDS_SECTORMASK			0x1ff //512
#define HEDS_POINTCLSTMSK		0x7f00
#define HEDS_FIRSTFATPOS		0x0003
#define HEDS_DIRCNTMASK			0x0f
#define HEDS_ENDCLUSTMSK		0xffff
#define HEDS_ENDFATMASK			0x7f
#define HEDS_FATSECSHIFT		8

//Offsets file info
#define HEDS_FILE_DIR_CLUST		26 //0x1a
#define HEDS_FILE_SIZE			28 //0x1c

//volume flags
//sector needs to be written
#define HEDS_VF_DIRTY		0x01

typedef struct _tagHEDskVolInfo
{
	DWORD	bootSector;
	DWORD	rootFat;
	DWORD	rootDir;
	DWORD	rootData;
	//Used by GetSector and SetSector for caching
	DWORD	curSector;
	//more economical to calc this once
	DWORD	bytesPerClust;
	WORD	dirEntries;
	//This is about writting a file so keep a free fat position
	WORD	freeFatPos;
	//Sectors per Cluster
	BYTE	secPerClust;
	//Flags, of course
	BYTE	volFlags;

} HEDskVolInfo, *pHEDskVolInfo;

typedef struct _tagHEDskFileInfo
{
	DWORD	size;
	//Current file pointer by cluster sectorPos and pointer
	DWORD	curSectorPos;
	//Always less than 512, pointer into curSector
	WORD	curPointer;
	//so we don't have to calc it
	WORD	curClust;
	//So it doesn't have to be looked up more than once
	//By WORD, assumes partition not too far out
	WORD	dirSect;
	WORD	dirPos;

} HEDskFileInfo, *pHEDskFileInfo;

//Initialize the drive information
bool HEDsk_LoadVolumeInfo( );

//Hunts for next avaliable free cluster and sets location in Volume info
void HEDsk_GetFreeFat( );

//add a new cluster to file
void HEDsk_AddCluster( pHEDskFileInfo pInfo );

//TODO HEDsk_CloseVolume(..) to flush last buffer if needed.
// Can be done with GetSectorData( 0 )

/*
	[in] Using cananonical name as "TEST    TXT" does not need trailing null
	[out] Populates diskInfo if file found
	Same as: OpenFile(...)
	does not search sub directories, file needs to exsist and be in root
*/
bool HEDsk_GetFileInfo( pHEDskFileInfo pInfo, char* lpsCanonicalName );

//This only sets the DskFileInfo and does not read the data sector
//Warning: No check here that pos is <= file size for now
void HEDsk_SeekFile( pHEDskFileInfo pInfo, DWORD pos );

//Read from the current file position, count to buf
bool HEDsk_ReadFile( pHEDskFileInfo pInfo, BYTE* buf, WORD count );

//Writes at the current file position and will alloc as file grows
bool HEDsk_WriteFile( pHEDskFileInfo pInfo, BYTE* buf, WORD count );

//Like 'File.Close' but info is still really open, just a flush
//Updates file size in dir. TODO ? Set the modified date ?
bool HEDsk_FlushFile( pHEDskFileInfo pInfo );

//need this for testing, delete for port
extern pHEDskVolInfo pVolInfo;

//not inlined in port, but this may not be the right thing to do
inline DWORD HEDsk_GetCurFileSector( pHEDskFileInfo pInfo )
{
	HEDskVolInfo& heVol= *pVolInfo;

	return
		heVol.rootData
		+ ( pInfo->curClust - 2 ) * (WORD)heVol.secPerClust //this should be shift, not call to mult(..)
		+ ( pInfo->curSectorPos & ( heVol.secPerClust - 1 ) );
}

