/*
Secure Digital Card Interface for the MSP430
written by
F. Foust
Dept. of Electrical and Computer Engineering
Michigan State University, 2004
*/

#include  "Device_430.h"
#include "HE_util.h"
#include "integer.h"
#include "spi.h"
#include "sd.h"

/* Initialize and enable the SPI module */
void spi_initialize( )
{
	SPI_PORT_SEL = SPI_PINS_SEL; // Setup Px for SPI mode
	SS_PORT_DIR|= SS_PIN; // Setup the SS as output
	// low. So, initialize it high.
	SS_PORT_OUT|= SS_PIN; // Set SS High
	
	SD_UCTL= (CHAR | SYNC | MM | SWRST); // 8-bit, SPI, Master
	SD_UTCTL= (SSEL1 | STC | CKPH); // Normal polarity, 3-wire
	SD_UBR0= 0x002; // SPICLK = SMCLK/2 (2=Minimum divisor)
	SD_UBR1= 0x000;
	SD_UMCTL= 0x000;
	MOD_REG|= SD_USPIE; // Module enable
	SD_UCTL&= ~SWRST; // SPI enable
}


/* Set the baud-rate divisor. The correct value is computed by dividing
the clock rate by the desired baud rate. The minimum divisor allowed
is 2. */
void spi_set_divisor(unsigned int divisor)
{
	SD_UCTL|= SWRST; // Temporarily disable the SPI module
	SD_UBR1= divisor >> 8;
	SD_UBR0= divisor;
	SD_UCTL&= ~SWRST; // Re-enable SPI
}

/* Assert the CS signal, active low (CS=0) */
inline void spi_cs_assert( )
{
	SS_PORT_OUT&= ~SS_PIN;
}

/* Deassert the CS signal (CS=1) */
inline void spi_cs_deassert( )
{
	SS_PORT_OUT|= SS_PIN;
}

/* Send a single byte over the SPI port */
void spi_send_byte( unsigned char input )
{
	SPI_IFG&= ~SD_UTXIFG;
	/* Send the byte */
	SD_UTXBUF= input;
	/* Wait for the byte to be sent */
	while ( ! ( SPI_IFG & SD_UTXIFG ) )
		;
}

/* Receive a byte. Output an 0xFF (the bus idles high) to receive the byte */
unsigned char spi_rcv_byte( )
{
	unsigned char tmp;
	SPI_IFG&= ~SD_URXIFG;
	
	/* Send the byte */
	SD_UTXBUF= 0xFF;
	
	/* Wait for the byte to be received */
	while( ! ( SPI_IFG & SD_URXIFG ) )
		;
	tmp= SD_URXBUF;
	return tmp;
}

/* Disable the SPI module. This function assumes the module had
* already been initialized. */
/*
void spi_disable( )
{
	// Put the SPI module in reset mode
	SD_UCTL|= SWRST;
	
	// Disable the USART module
	MOD_REG&= ~SD_USPIE;
}
*/
/*
void spi_enable( )
{
	// Enable the USART module
	MOD_REG|= SD_USPIE;
	
	// Take the SPI module out of reset mode
	SD_UCTL&= ~SWRST;
}
*/
