// mfc_diskioView.cpp : implementation of the Cmfc_diskioView class
//

#include "stdafx.h"
#include <winioctl.h>
#include "controls.h"

#include "HE_Dos.h"
#include "HE_Drives.h"
#include "mfc_diskio.h"

#include "mfc_diskioDoc.h"
#include ".\mfc_diskioview.h"

#include "dosfs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//global drive to start port
pHEDriveInfo pGDrvInfo;

//global sector buffer to start port
BYTE secBuf[ 512 ];

//global volume info to start port
HEDskVolInfo* pVolInfo;

//for the DFS code
uint32_t DFS_ReadSector( uint8_t unit, uint8_t *buf, DWORD sector, DWORD count )
{
	return ! GetSectorData( pGDrvInfo, sector, buf );
}

uint32_t DFS_WriteSector(uint8_t unit, uint8_t *buffer, uint32_t sector, uint32_t count)
{
	return ! SetSectorData( pGDrvInfo, sector, buffer );
}

//Form for the target micro
bool GetSectorData( DWORD sector )
{
	//This is done to make the port easier
	HEDskVolInfo& heVol= *pVolInfo;

	//cached
	if( heVol.curSector == sector )
		return true;

	if( heVol.volFlags & HEDS_VF_DIRTY
		&& ! SetSectorData( pGDrvInfo, heVol.curSector, secBuf )
	)
		return false;

	heVol.curSector= sector;
	//clear dirty
	heVol.volFlags&= ~HEDS_VF_DIRTY;

	return GetSectorData( pGDrvInfo, sector, secBuf );
}

//Form for the target micro
bool SetSectorData( DWORD sector )
{
	//This is done to make the port easier
	HEDskVolInfo& heVol= *pVolInfo;

	if( heVol.curSector == sector )
	{
		//just mark it
		heVol.volFlags|= HEDS_VF_DIRTY;
		return true;
	}
	//else a new sector, was the last one dirty?
	if( heVol.volFlags & HEDS_VF_DIRTY )
	{
		if( ! SetSectorData( heVol.curSector ) )
			return false;
	}
	heVol.curSector= sector;

	//Note that the compiler will put this and the
	// other 'just mark it' together with a jump
	heVol.volFlags|= HEDS_VF_DIRTY;
	return true;
}

// Cmfc_diskioView
IMPLEMENT_DYNCREATE(Cmfc_diskioView, CEditView)

BEGIN_MESSAGE_MAP(Cmfc_diskioView, CEditView)
	ON_WM_PAINT( )
	ON_EN_CHANGE( IDC_DB_SECTOR, OnDBSectorChange )
	ON_CBN_SELCHANGE( IDC_DB_DRIVES, OnCbnSelchangeDbDrives )

	// Standard printing commands
	ON_COMMAND( ID_FILE_PRINT, CEditView::OnFilePrint )
	ON_COMMAND( ID_FILE_PRINT_DIRECT, CEditView::OnFilePrint )
	ON_COMMAND( ID_FILE_PRINT_PREVIEW, CEditView::OnFilePrintPreview )
END_MESSAGE_MAP( )

Cmfc_diskioView::Cmfc_diskioView( )
	:pCurDrive( NULL )
{
}

Cmfc_diskioView::~Cmfc_diskioView( )
{
}

void Cmfc_diskioView::OnInitialUpdate( )
{
	if( ! barDisk.Create( dynamic_cast<CFrameWnd*>( AfxGetMainWnd( ) ),
		IDD_DIALOGBAR_DISK, WS_VISIBLE | CBRS_TOP | CBRS_GRIPPER
		| CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC, IDD_DIALOGBAR_DISK ) )//AFX_IDW_RESIZE_BAR + 1 ) )
	{
		TRACE( "Failed to create dialog bar barDisk\n" );
		return; // fail to create
	}

	barDisk.SetNotify( this );
	if( ! cBarEditSector.SubclassDlgItem( IDC_DB_SECTOR, &barDisk ) )
	{
		TRACE( "Failed to subclass IDC_DB_SECTOR\n" );
		return; // fail to create
	}

	if( ! cBarComboDrives.SubclassDlgItem( IDC_DB_DRIVES, &barDisk ) )
	{
		TRACE( "Failed to subclass IDC_DB_SECTOR\n" );
		return; // fail to create
	}
	CEditView::OnInitialUpdate( );

	EnumDrives( set );

	long i= 0;
	for( HEDriveInfoSetIt it= set.begin( ); it != set.end( ); ++it )
	{
		CString strT;
		strT.Format( _T("%s[ %s:%s ]"), it->strLogicalName, it->strDriveName, it->strFSType );
		cBarComboDrives.InsertString( i, strT );
		cBarComboDrives.SetItemData( i, (DWORD_PTR)&*it );
		++i;
	}

	VERIFY( font.CreateFont(
		14,                        // nHeight
		0,                         // nWidth
		0,                         // nEscapement
		0,                         // nOrientation
		FW_BOLD,                   // nWeight
		FALSE,                     // bItalic
		FALSE,                     // bUnderline
		0,                         // cStrikeOut
		ANSI_CHARSET,              // nCharSet
		OUT_DEFAULT_PRECIS,        // nOutPrecision
		CLIP_DEFAULT_PRECIS,       // nClipPrecision
		DEFAULT_QUALITY,           // nQuality
		DEFAULT_PITCH | FF_SWISS,  // nPitchAndFamily
		"Courier"                  // lpszFacename	GetEditCtrl( ).SetFont( &font );
	) );
	GetEditCtrl( ).SetFont( &font );


/*


	The test .............................


*/

//Testing this here because it just runs right away....
//we know that the sd card is drive one on this machine
	//Drives come and go on my machine so the ordinal
	// position is not reliable...
	for( HEDriveInfoSetIt it= set.begin( ); it != set.end( ); ++it )
	{
//  **** Put the name of your sd drive here!  **** 
		if( it->strDriveName == _T("TEST_MSP430") )
		{
			long dpos= it - set.begin( );
			cBarComboDrives.SetCurSel( dpos );
			pCurDrive= &set.at( dpos );
			break;
		}
	}

//testing, need global for DFS_...
	pGDrvInfo= pCurDrive;

	HEDskVolInfo heVol;

	//set the global pointer for HEDsk code
	pVolInfo= &heVol;

	//Load the Disk Volume Information
	HEDsk_LoadVolumeInfo( );

	HEDskFileInfo fileInfo;

	//READ test
	// "test.txt", the test file...
	//if( HEDsk_GetFileInfo( &fileInfo, _T("TEST    TXT") ) )
	//{
	//	//read some of the file, cross clusters
	//	//easy to compare the files in notepad++ to
	//	//make sure the data matches, needs some 100k of text for this

	//	HEDsk_SeekFile( &fileInfo, 32500 );
	//	char buf[ 35000 ];
	//	HEDsk_ReadFile( &fileInfo, (BYTE*)buf, 35000 );

	//	//test.out will be in the VS project directory
	//	CFile of( "test.out", CFile::modeCreate | CFile::modeWrite );
	//	of.Write( buf, 35000 );
	//	of.Close( );

	//}
	//else
	//	ASSERT( false );


	//WRITE test
	//Make sure this file exsists
	if( HEDsk_GetFileInfo( &fileInfo, _T("TEST    TXT") ) )
	{
		//char buf[ 500 ];
		//HEDsk_ReadFile( &fileInfo, (BYTE*)buf, 300 );
		//HEDsk_SeekFile( &fileInfo, 511 );

		for( long t= 0; t < 50; ++t )
		{
			CString strT;
			strT.Format( _T("This is text that will be written to the file: %d\r\n"), t );
			HEDsk_WriteFile( &fileInfo, (BYTE*)(LPCTSTR)strT, strT.GetLength( ) );
		}
		HEDsk_FlushFile( &fileInfo );
	}
	else
		ASSERT( false );

	CString strT;
	strT.Format( _T("%xh"), heVol.bootSector );
	cBarEditSector.SetWindowText( strT );
/*
	Use to test DOSFS code included in this project

	VOLINFO vinfo;
	//BYTE s1;
	//DIRINFO dirinfo;
	DWORD s4;
	//FILEINFO fileinfo;
	//char fileBuf[ 1000 ];
	char fbWrite[]= "\r\nthis is a test line from mfc_diskio\r\n";
	DWORD fbWriteSize= (DWORD)strlen( fbWrite );

	DFS_GetVolInfo( 0, sector, 0x89, &vinfo );
	DFS_GetFAT( &vinfo, sector, &s4, 0 );

	//DFS_OpenDir( &vinfo, (uint8_t*)"/", &dirinfo );

	//DFS_OpenFile( &vinfo, (uint8_t*)"/test.txt", DFS_READ, sector, &fileinfo );
	//DFS_Seek( &fileinfo, fileinfo.filelen - 1, sector );
	//DFS_ReadFile( &fileinfo, sector, (uint8_t*)fileBuf, &s4, 512 );

	//FILEINFO wfileinfo;
	//DFS_OpenFile( &vinfo, (uint8_t*)"/test.txt", DFS_WRITE, sector, &wfileinfo );
	//DFS_Seek( &wfileinfo, wfileinfo.filelen, sector );
	//DFS_WriteFile( &wfileinfo, sector, (uint8_t*)fbWrite,&s4, fbWriteSize );

*/
}

void Cmfc_diskioView::OnDBSectorChange( )
{
	if( ! pCurDrive )
		return;

	TCHAR* p;
	CString strT;
	cBarEditSector.GetWindowText( strT );
	curSector= _tcstol( strT, &p, 16 );

	bool bOk= GetSectorData( pCurDrive, curSector, sector );

	if( bOk )
		SetViewSectorData( sector );
}

void Cmfc_diskioView::OnCbnSelchangeDbDrives( )
{
	long pos= cBarComboDrives.GetCurSel( );
	pCurDrive= &set.at( pos );
	//Read MBR on a disk change
	cBarEditSector.SetWindowText( _T("0h") );
}

void Cmfc_diskioView::SetViewSectorData( BYTE* pBytes )
{
	GetEditCtrl( ).SetSel( 0, -1 );
	GetEditCtrl( ).Clear( );

	CString strT;
	strT= _T("       0   1   2   3   4   5   6   7   8   9   a   b   c   d   e   f\r\n\r\n");
	GetEditCtrl( ).SetSel( -1, -1 );
	GetEditCtrl( ).ReplaceSel( strT );

	for( long i= 0; i < 512; i+= 16 )
	{
		TCHAR buf[ 22 ];
		//TCHAR buf[ ]= "test";
		memcpy( buf, (TCHAR*)&pBytes[ i ], sizeof( TCHAR) * 16 );
		for( long t= 0; t < 16; ++t )
			if( buf[ t ] == _T('\0') || buf[ t ] == _T('\n') || buf[ t ] == _T('\r') || buf[ t ] == _T('\t') )
				buf[ t ]= _T(' ');

		buf[ 16 ]= _T('\0');

		strT.Format( _T("%.4x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x  %.2x   %s\r\n"),
			i,
			pBytes[ i ],
			pBytes[ i + 1 ],
			pBytes[ i + 2 ],
			pBytes[ i + 3 ],
			pBytes[ i + 4 ],
			pBytes[ i + 5 ],
			pBytes[ i + 6 ],
			pBytes[ i + 7 ],
			pBytes[ i + 8 ],
			pBytes[ i + 9 ],
			pBytes[ i + 10 ],
			pBytes[ i + 11 ],
			pBytes[ i + 12 ],
			pBytes[ i + 13 ],
			pBytes[ i + 14 ],
			pBytes[ i + 15 ],
			buf
		);
		GetEditCtrl( ).SetSel( -1, -1 );
		GetEditCtrl( ).ReplaceSel( strT );
	}

	strT.Format( _T("(stuff)\r\n\r\n") );
	GetEditCtrl( ).SetSel( -1, -1 );
	GetEditCtrl( ).ReplaceSel( strT );
}

BOOL Cmfc_diskioView::PreCreateWindow( CREATESTRUCT& cs )
{
	cs.style|= ES_MULTILINE;
	cs.style &= ~WS_HSCROLL;
	//return TRUE;
	return CEditView::PreCreateWindow( cs );
}

void Cmfc_diskioView::OnPaint( )
{
	CEditView::OnPaint( );
}

void Cmfc_diskioView::OnDraw( CDC* pDC )
{
	CEditView::OnDraw( pDC );
}

// Cmfc_diskioView printing .....................................................
BOOL Cmfc_diskioView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return DoPreparePrinting(pInfo);
}

void Cmfc_diskioView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void Cmfc_diskioView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

#ifdef _DEBUG
void Cmfc_diskioView::AssertValid() const
{
	CEditView::AssertValid();
}

void Cmfc_diskioView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

Cmfc_diskioDoc* Cmfc_diskioView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(Cmfc_diskioDoc)));
	return (Cmfc_diskioDoc*)m_pDocument;
}
#endif //_DEBUG

